﻿using MindFusion.Diagramming;
using MindFusion.Drawing;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace FractalLayout
{
	// Learn more about making custom code visible in the Xamarin.Forms previewer
	// by visiting https://aka.ms/xamarinforms-previewer
	[DesignTimeVisible(false)]
	public partial class MainPage : ContentPage
	{
		public MainPage()
		{
			InitializeComponent();

			diagram = diagramView.Diagram;

			bounds = new Rectangle(0, 0, 10, 8);

			brushes = new Brush[] {
				new LinearGradientBrush(Colors.LightSteelBlue, Colors.BlueViolet, 0),
				new LinearGradientBrush(Colors.White, Colors.LightBlue, 0),
				new LinearGradientBrush(Colors.White, Colors.DeepSkyBlue, 0),
				new LinearGradientBrush(Colors.LimeGreen, Colors.Green, 0)
			};

			random = new Random();
		}

		private void OnRandomClick(object sender, EventArgs e)
		{
			diagram.ClearAll();

			ShapeNode root = diagram.Factory.CreateShapeNode(bounds);
			RandomTree(root, 5, 4);
			Arrange(root);
		}

		void Arrange(DiagramNode root)
		{
			var layout = new MindFusion.Diagramming.Layout.FractalLayout();
			layout.Root = root;
			layout.Arrange(diagram);
			diagram.ResizeToFitItems(0);
			diagramView.ZoomToFit();
			diagram.ZoomFactor *= 2;   
		}

		void RandomTree(DiagramNode node, int depth, int minChildren)
		{
			if (depth <= 0)
				return;

			Diagram diagram = node.Parent;
			int children = random.Next(depth) - 1 + minChildren;

			if (diagram.Nodes.Count < 3 && children < 2)
				children = 2;

			for (int i = 0; i < children; ++i)
			{
				// create child node and link
				ShapeNode child = diagram.Factory.CreateShapeNode(bounds);
				child.Brush = brushes[depth % brushes.Length];
				diagram.Factory.CreateDiagramLink(node, child)
					.HeadShape = null;

				// build child branch
				RandomTree(child, depth - 1, minChildren);
			}
		}

		Diagram diagram;
		readonly Rectangle bounds;
		readonly Brush[] brushes;
		readonly Random random;
	}
}
